@@ -62,8 +62,31 @@ class UniprotAPI:
         # by the minidom XML parser
         dom = self.getDOMObject(accessionID)
 
+
         return self._build_and_complete(accessionID, dom, datastore)
 
+    
+    def sideload_from_file(self, datastore, dom):
+        
+        childNodes = dom.childNodes
+        
+        accession = ""
+
+        for i in childNodes:
+            try:
+                if str(i.nodeName) == "accession":
+                    accession = str(i.childNodes[0].nodeValue).upper()
+                    break
+            except:
+                continue
+            
+        if accession == "":
+            print "Unable to find accession tag"
+            return False
+
+        return(self._build_and_complete(accession, dom, datastore))
+
+                
 
     
     def _build_and_complete(self, accessionID, dom, datastore):
@@ -77,7 +100,7 @@ class UniprotAPI:
         try:
             xml = dom.toxml()
         except AttributeError:
-            print "Networking error - unable to retrieve " + accessionID + " from the UniProt server"
+            print "Error - unable to parse xml associated with " + accessionID
             return failure
 
         # If we get here then the networking portion theoretically worked
@@ -94,6 +117,7 @@ class UniprotAPI:
             other_accessions = self._getOtherAccessionValues(dom, accessionID)
             species = self._getSpecies(dom, accessionID)
             domains = self._getDomains(sequence, accessionID)
+            host = self._getHost(dom, accessionID)
             taxonomy = self._getTaxonomy(dom, accessionID)
             isoforms = self._getIsoforms(dom, accessionID)
         except UniprotAPIException, e:
@@ -106,11 +130,11 @@ class UniprotAPI:
         # so, if we got here were able to parse the XML ok, so jobs a good'un!
         # return object
         if datastore == -1:
-            return ProteinObject.ProteinObject(accessionID, version, xml, name, mutations, sequence, creationDate, geneID, geneName, other_accessions, species, domains, taxonomy, isoforms, "UniProt")
+            return ProteinObject.ProteinObject(accessionID, version, xml, name, mutations, sequence, creationDate, geneID, geneName, other_accessions, species, domains, taxonomy, isoforms, "UniProt", host)
 
         # update datatstore and return silently
         else:
-            datastore[accessionID] = ProteinObject.ProteinObject(accessionID, version, xml, name, mutations, sequence, creationDate, geneID, geneName, other_accessions, species, domains, taxonomy, isoforms, "UniProt")
+            datastore[accessionID] = ProteinObject.ProteinObject(accessionID, version, xml, name, mutations, sequence, creationDate, geneID, geneName, other_accessions, species, domains, taxonomy, isoforms, "UniProt", host)
 
 
 #--------------------------------------------------------
@@ -269,7 +293,7 @@ class UniprotAPI:
 #--------------------------------------------------------
 # PRIVATE FUNCTION
 #--------------------------------------------------------
-# Internal function which gets the protein sequence
+# Internal function which gets the protein version
 
     def _getVersion(self, domObject, ID):
         found = False
@@ -472,17 +496,16 @@ class UniprotAPI:
         for element in elementList:
             
             # ________________________________________________________________
-            # look for root XML based "assession" values
+            # look for root XML based "accession" values
             if element.nodeName == 'accession':
                 tempID = element.childNodes[0].nodeValue
-                accList.append((tempID, ProteinParser.ID_type(tempID)[1]))
+                accList.append((str(tempID), ProteinParser.ID_type(tempID)[1]))
 
             # ________________________________________________________________
             # also look for a 'name' XML value, add as Swissprot as we do in
             # the equivalent function in ProteinObject.py 
             elif element.nodeName == 'name':
-                accList.append((element.childNodes[0].nodeValue, "Swissprot"))
-
+                accList.append((str(element.childNodes[0].nodeValue), "Swissprot-Locus"))
                 
             # ________________________________________________________________
             # Look for EMBL protein sequences
@@ -492,18 +515,18 @@ class UniprotAPI:
                 for subElement in element.childNodes:
                     if subElement.nodeType == subElement.ELEMENT_NODE and subElement.getAttribute('type') == "protein sequence ID":
                         tempID = subElement.getAttribute('value')
-                        accList.append((tempID, ProteinParser.ID_type(tempID)[1]))
+                        accList.append((str(tempID), ProteinParser.ID_type(tempID)[1]))
 
                         
             # ________________________________________________________________
             # Finally also look for refseq and IPI are a little accession valyes
-            elif element.nodeType == element.ELEMENT_NODE and element.getAttribute("type") in ["IPI", "RefSeq"]:
+            elif element.nodeType == element.ELEMENT_NODE and element.getAttribute("type") in ("IPI", "RefSeq"):
                 tempID = element.getAttribute('id')
-                accList.append((tempID, ProteinParser.ID_type(tempID)[1]))
+                accList.append((str(tempID), ProteinParser.ID_type(tempID)[1]))
 
         return list(set(accList))
 
-#--------------------------------------------------------
+#-------------------------------------------------------
 # PRIVATE FUNCTION
 #--------------------------------------------------------
 # Internal function which gets the species from which
@@ -519,6 +542,36 @@ class UniprotAPI:
                     if subElement.nodeType == subElement.ELEMENT_NODE and subElement.getAttribute("type") == "scientific":
                         return (subElement.childNodes[0].nodeValue)
 
+
+#-------------------------------------------------------
+# PRIVATE FUNCTION
+#--------------------------------------------------------
+# Internal function which gets the species from which
+# this protein is derived
+#
+
+    def _getHost(self, domObject, ID):
+        
+        scientificName = ""
+        commonName = ""
+        
+        hostnames = domObject.getElementsByTagName("organismHost")
+
+        if hostnames: 
+            for subElement in hostnames[0].childNodes:
+                if subElement.nodeType == subElement.ELEMENT_NODE and subElement.getAttribute("type") == "scientific":
+                    scientificName = subElement.childNodes[0].nodeValue
+                if subElement.nodeType == subElement.ELEMENT_NODE and subElement.getAttribute("type") == "common":
+                    commonName = subElement.childNodes[0].nodeValue
+        else:
+            return "N/A"
+
+        if scientificName == "" and commonName == "":
+            raise UniprotAPIException("Error - unable to find host organism name despite 'organismHost' tags in ID " + ID)
+        else:
+            return str(scientificName + " (" + commonName + ")") 
+
+
 #--------------------------------------------------------
 # PRIVATE FUNCTION
 #--------------------------------------------------------
@@ -535,13 +588,13 @@ class UniprotAPI:
         # class-space polution
         def getPfamDOM(ID):
             handle = self.Network.PfamNetworkRequest(ID, False)
-        
+            
             try:
                 domObject = parseString(handle.read())
             except Exception, e:
                 print e 
                 raise UniprotAPIException("Error when converting Pfam XML to DOM object")
-        
+            
             return domObject
         #===========================================================
         
@@ -562,13 +615,18 @@ class UniprotAPI:
             # will miss align!
             if element.nodeName == 'sequence':
 
-                if not len(element.childNodes[0].nodeValue) == len(sequence):
-                    print "Length difference between sequences"
-                    print "Pfam sequence (" + str(len(element.childNodes[0].nodeValue)) +") = " + str(element.childNodes[0].nodeValue).lower()
-                    print "Uniprot sequence ("  + str(len(sequence)) +") = " + str(sequence).lower()
+                try:
                     
-                    raise UniprotAPIException("Pfam and Uniprot sequences fail to match for accession " + ID)
-
+                    if not len(str(element.childNodes[0].nodeValue)) == len(sequence):
+                        print "Length difference between sequences"
+                        print "Pfam sequence (" + str(len(element.childNodes[0].nodeValue)) +") = " + str(element.childNodes[0].nodeValue).lower()
+                        print "Uniprot sequence ("  + str(len(sequence)) +") = " + str(sequence).lower()
+                    
+                        raise UniprotAPIException("Pfam and Uniprot sequences fail to match for accession " + ID)
+                except Exception, e:
+                    print e
+                    # print element.toxml()
+                    return element
             elif element.nodeName == 'matches':
                 for match in element.childNodes:
                     if match.nodeName == 'match':
